// SchemeDocument.m


#import "SchemeDocument.h"


extern int eval_scheme(char *, SchemeDocument *);
extern char *get_result();
extern NSString *prepare_loaded_data(NSString *);
extern void register_document(SchemeDocument *);
extern void unregister_document(SchemeDocument *);
extern void idle(SchemeDocument *);


static BOOL startup_file_loaded = NO;
static BOOL timer_installed = NO;


@implementation SchemeDocument


- (id)init
{
  if((self = [super init])) {
    error = NO;
    register_document(self);
  }

  return self;
}


- (void)dealloc
{
  unregister_document(self);
  [super dealloc];
}


- (NSString *)string { return string; }
- (NSTextView *)textView { return textView; }
- (NSString *)windowNibName { return @"SchemeDocument"; }


- (void)setString: (NSString *)s
{
  [s retain];
  [string release];
  string = s;
}


- (void)updateView { [textView setString: [self string]]; }
- (void)updateString { [self setString: [textView string]]; }


- (void)windowControllerDidLoadNib:(NSWindowController *) aController
{
  [super windowControllerDidLoadNib:aController];
  
  if(!string) {
    [self setString: @""];

    if(!startup_file_loaded) {
      startup_file_loaded = YES;
      
      if(![self evalString: @"(CHICKEN:startup)"])
	[self setString: [NSString stringWithCString: get_result()]];
    }

    if(!timer_installed) {
      timer_installed = YES;
      [NSTimer scheduledTimerWithTimeInterval: 0.1 target: self selector: @selector(idle:) userInfo: nil repeats: YES];
    }
  }

  [textView setFont: [NSFont fontWithName: @"Courier" size: 14]];
  /*
  [textView setBackgroundColor: [NSColor colorWithCalibratedRed: 0.1373 green: 0.2314 blue: 0.3529 alpha: 1.0]];
  [textView setTextColor: [NSColor whiteColor]];
  */
  [self updateView];
}


- (NSData *)dataRepresentationOfType:(NSString *)aType
{
  [self updateString];
  return [string dataUsingEncoding: NSASCIIStringEncoding];
}


- (BOOL)loadDataRepresentation:(NSData *)data ofType:(NSString *)aType
{
  NSString *s = [[NSString alloc] initWithData: data encoding: NSASCIIStringEncoding];
  s = prepare_loaded_data(s);
  [self setString: s];
  [s release];
  [self updateView];
  return YES;
}


- (void)textDidChange: (NSNotification *)n
{
  [self updateChangeCount: NSChangeDone];
}


- (BOOL)evalString: (NSString *)str
{
  char *buf = (char *)[str cString];
  return eval_scheme(buf, self);
}


- (char *)evalSelection
{
  NSRange r = [textView selectedRange];
  char *buf = NULL;

  if(r.length > 0) {
    buf = (char *)malloc(r.length + 1);
    [[textView string] getCString: buf maxLength: r.length range: r remainingRange: nil];
    buf[ r.length ] = '\0';
    error = !eval_scheme(buf, self);
    return get_result();
  }

  return buf;
}


- (void)insertAndSelect: (char *)buf
{
  if(buf) {
    NSRange r = [textView selectedRange], r2;
    r.location += r.length;
    r.length = 0;
    r2.location = r.location;
    r2.length = strlen(buf);
    [textView setSelectedRange: r];
    [textView insertText: [NSString stringWithCString: buf]];
    [textView setSelectedRange: r2];
  }
}


- (IBAction)eval: (id)sender { [self insertAndSelect: [self evalSelection]]; }


- (IBAction)evalAndReplace: (id)sender
{
  char *buf = [self evalSelection];
 
  if(buf) {
    if(error) [self insertAndSelect: buf];
    else {
      NSRange r = [textView selectedRange];
      [textView insertText: [NSString stringWithCString: buf]];
      r.length = strlen(buf);
      [textView setSelectedRange: r];
    }
  }
}


- (BOOL)validateMenuItem: (NSMenuItem *)item
{
  return YES;
  //return [textView selectedRange].length > 0;
}


- (void)idle: (NSTimer *)note
{
  //NSLog(@"%@ is idle...", self);
  idle(self);
}


@end
