;;;; stub.scm


#>
#import <Cocoa/Cocoa.h>
#import "SchemeDocument.h"
<#

(use srfi-1)

(define-constant +startup-file+ "~/.CHICKEN")

(define *last-result* #f)
(define printer pp)
(define current-document (make-parameter #f))
(define *documents* '())

(define-external (register_document ((pointer "SchemeDocument") doc)) void
  (set! *documents* (cons doc *documents*)) )

(define-external (unregister_document ((pointer "SchemeDocument") doc)) void
  (set! *documents* (delete doc *documents* equal?)) )

(define-external (eval_scheme (c-string str) ((pointer "SchemeDocument") doc)) bool
  (##sys#clear-trace-buffer)
  (current-document doc)
  (handle-exceptions ex 
      (begin
	(set! *last-result*
	  (call-with-output-string
	   (lambda (o)
	     (print-error-message ex o)
	     (print-call-chain o) ) ) )
	#f)
    (set! *last-result*
      (with-output-to-string
	(lambda ()
	  (printer (eval (with-input-from-string (conc "(begin " str "\n)") read)) ) ) ) )
    #t) )

(define-external (get_result) c-string
  (##sys#make-c-string *last-result*) )

#>!
static void nslog(char *str)
{
 NSLog([NSString stringWithCString: str]);
}

static ___discard char *nsstring_to_string(NSString *s)
{
  int len = [s length];
  char *c = (char *)malloc(len + 1);
  
  memcpy(c, [s cString], len);
  c[ len ] = '\0';
  [s release];
  return c;
}

static NSString *string_to_nsstring(char *s)
{
  return [[NSString alloc] initWithCString: s];
}

static void insert_text(SchemeDocument *w, char *txt)
{
  [[w textView] insertText: [NSString stringWithCString: txt]];
}

static void notify(SchemeDocument *w)
{
 NSNotification* myNotification = 
   [NSNotification notificationWithName: @"SchemeDocumentIdle" object: w];

 [[NSNotificationQueue defaultQueue] enqueueNotification: myNotification postingStyle: NSPostWhenIdle]; 
}
<#

(define-external (prepare_loaded_data ((pointer "NSString") s)) (pointer "NSString")
  (string_to_nsstring (untabbify (nsstring_to_string s))) )

(define (untabbify s)
  (let ((len (string-length s)))
    (let loop ((i 0) (p 0) (c 0) (acc '()))
      (if (fx>= i len)
	  (string-intersperse (reverse (cons (substring s p len) acc)) "")
	  (let ((ch (string-ref s i)))
	    (case ch
	      ((#\newline)
	       (loop (fx+ i 1) p 0 acc) )
	      ((#\tab) 
	       (let ((n (- 8 (fxmod c 8))))
		 (loop (fx+ i 1) (fx+ i 1) (fx+ c n) (cons* (make-string n #\space) (substring s p i) acc))))
	      (else (loop (fx+ i 1) p (fx+ c 1) acc)) ) ) ) ) ) )

(define (insert-text txt #!optional (doc (current-document)))
  (and doc
       (insert_text doc txt) ) )

(define (CHICKEN:startup)
  (when (file-exists? +startup-file+) 
    (load +startup-file+) ) )

(define-external (idle ((pointer "SchemeDocument") doc)) void
  (void) )

(return-to-host)
